/****************************************************************
 * ItalkServer.cc : Italk-Server status class
 *
 * (c) 1997 Yutaka Oiwa.
 *
 * $Id: ItalkServer.cc,v 1.6 1998/09/23 17:51:26 yutaka Exp $
 ****************************************************************/

#include <stdlib.h>
#include "Defines.h"
#include "ItalkServer.h"

ItalkServer::ItalkServer(const char *name_, int port_, const char* fullname_,
			 ItalkServer_callback_t callback_, void *app_data_) {
  name = NewString(name_);
  port = port_;
  fullname = NewString(fullname_);
  callback = callback_;
  app_data = app_data_;

  add_to = 1;

  for (int i=0; i < 2; i++) {
    n_monkeys[i] = 0;
    message[i] = 0;
    mes_urgent[i] = 0;
    for (int j=0; j <= MAX_MONKEY; j++) {
      monkeys[i][j] = NULL;
      // monkeys[i][MAX_MONKEY] is a guardian.
    }
  }
}

const Monkey *const * ItalkServer::GetMonkeys(void) const {
  return monkeys[!add_to];
}

void ItalkServer::AddMonkey(Monkey *p) {
  if (n_monkeys[add_to] < MAX_MONKEY) {
    monkeys[add_to][n_monkeys[add_to]++] = p;
  } else {
    n_monkeys[add_to]++;
    delete p;
  }
}

void ItalkServer::AddMessage(const char *p, int urgent) {
  dprintf(8, "MESSAGE: %s\n", p);
  if (message[add_to])
    delete[] message;
  message[add_to] = NewString(p);
  mes_urgent[add_to] = urgent;
}

void ItalkServer::DeleteMessage() {
  dprintf(8, "MESSAGE: removed\n");
  if (message[add_to])
    delete[] message[add_to];
  message[add_to] = NULL;
  mes_urgent[add_to] = 0;
}

void ItalkServer::DataFinished() {
  int f = (n_monkeys[add_to] > n_monkeys[!add_to]) || mes_urgent[add_to];
  add_to = !add_to;
  
  n_monkeys[add_to] = 0;
  for(int i=0; i < MAX_MONKEY; i++)
    if (monkeys[add_to][i]) {
      delete monkeys[add_to][i];
      monkeys[add_to][i] = NULL;
    }
  if (message[add_to])
    delete[] message[add_to];
  message[add_to] = NULL;
  mes_urgent[add_to] = 0;

  if (callback)
    callback(f, app_data);
}
